﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using Composite.Core.Routing;
using Composite.Core.WebClient.Media;
using Composite.Data;
using Composite.Data.Plugins.DataProvider;
using Composite.Data.Types;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;
using Microsoft.Practices.ObjectBuilder;

namespace CustomMediaFileProvider
{
    [ConfigurationElementType(typeof(MediaFileProviderData))]
    public abstract class MediaFileProvider: IDataProvider
    {
        private readonly string _providerName;
        private readonly string _storeId;
        private readonly string _elementTitle;
        private readonly string _elementDescription;
        private MediaFileStore _store;
        private readonly IMediaUrlProvider _mediaUrlProvider;
        private IEnumerable<UrlBasedMediaFile> _files;

        private List<MediaFileFolder> _folders;

        public IEnumerable<UrlBasedMediaFile> Files
        {
            set { _files = value; }
        }

        public DataProviderContext Context { set; private get; }
        public IEnumerable<Type> GetSupportedInterfaces()
        {
            return new[] { typeof(IMediaFile), typeof(IMediaFileFolder), typeof(IMediaFileStore) };
        }

        public MediaFileProvider(string providerName, string storeId, string title, string description, ResizingOptions resizingOptions = null)
        {
            _providerName = providerName;
            _storeId = storeId;

            _elementTitle = title;
            _elementDescription = description;

            _files = null;
            _folders = null;

            _mediaUrlProvider = (resizingOptions != null ) ? (IMediaUrlProvider) new ResizableMediaUrlProvider(this,resizingOptions): new MediaUrlProvider(this);

            MediaUrls.RegisterMediaUrlProvider(storeId, _mediaUrlProvider);
        }

        public IQueryable<T> GetData<T>() where T : class, IData
        {
            if (typeof (T) == typeof (IMediaFileStore))
            {
                return GetStoreQueryable() as IQueryable<T>;
            }

            if (typeof(T) == typeof(IMediaFile))
            {
                return GetFileQueryable() as IQueryable<T>;
            }

            if (typeof(T) == typeof(IMediaFileFolder))
            {
                return GetFolderQueryable() as IQueryable<T>;
            }

            throw new InvalidOperationException("Unexpected type parameter: " + typeof(T).FullName);
        }

        public T GetData<T>(IDataId dataId) where T : class, IData
        {
            var mediaDataId = dataId as Composite.Plugins.Data.DataProviders.MediaFileProvider.MediaFileProvider.MediaDataId;
            if (mediaDataId == null)
            {
                throw new InvalidOperationException();
            }

            if (mediaDataId.MediaType == Composite.Plugins.Data.DataProviders.MediaFileProvider.MediaFileProvider.MediaElementType.Store)
            {
                return _store as T;
            }

            if (mediaDataId.MediaType == Composite.Plugins.Data.DataProviders.MediaFileProvider.MediaFileProvider.MediaElementType.File)
            {
                var file = _files.FirstOrDefault(f => f.Id == mediaDataId.Id);
                return file as T;
            }

            if (mediaDataId.MediaType == Composite.Plugins.Data.DataProviders.MediaFileProvider.MediaFileProvider.MediaElementType.Folder)
            {
                var file = _folders.FirstOrDefault(f => f.Id == mediaDataId.Id);
                return file as T;
            }

            
            throw new InvalidOperationException();
        }

        private IQueryable<IMediaFileStore> GetStoreQueryable()
        {
            if (_store == null)
            {
                _store = new MediaFileStore(_storeId, _elementTitle, _elementDescription,
                    Context.CreateDataSourceId(new Composite.Plugins.Data.DataProviders.MediaFileProvider.MediaFileProvider.MediaDataId { MediaType = Composite.Plugins.Data.DataProviders.MediaFileProvider.MediaFileProvider.MediaElementType.Store }, typeof(IMediaFileStore)));
            }

            return (new[] { _store }).AsQueryable();
        }

        private IQueryable<IMediaFile> GetFileQueryable()
        {
            return _files.AsQueryable();
        }

        public UrlBasedMediaFile GetMediaFile(string fileName, string path, string downloadUrl)
        {
            //var dataId = new MediaDataId { FileName = fileName, Path = path, MediaType = ElementType.File};

            var dataId = new Composite.Plugins.Data.DataProviders.MediaFileProvider.MediaFileProvider.MediaDataId
            {
                Id = UrlBasedMediaFile.CalculateId(path, fileName),
                MediaType = Composite.Plugins.Data.DataProviders.MediaFileProvider.MediaFileProvider.MediaElementType.File
            };

            if (_folders == null || _folders.All(f => f.Path != path))
            {
                if(_folders==null)
                    _folders = new List<MediaFileFolder>();
                _folders.Add(GetMediaFolder(path));
            }

            return new UrlBasedMediaFile(
                        downloadUrl, fileName, path, _storeId, 1000,
                        DateTime.Now, DateTime.Now, new DataSourceId(dataId, _providerName, typeof (IMediaFile)));
        }

        public MediaFileFolder GetMediaFolder(string path)
        {
            var dataId = new Composite.Plugins.Data.DataProviders.MediaFileProvider.MediaFileProvider.MediaDataId
            {
                Id = UrlBasedMediaFile.CalculateId(path, ""),
                MediaType = Composite.Plugins.Data.DataProviders.MediaFileProvider.MediaFileProvider.MediaElementType.Folder
            };

            return new MediaFileFolder(path, _storeId, dataId, _providerName);
        }

        private IQueryable<IMediaFileFolder> GetFolderQueryable()
        {
            return _folders.AsQueryable();
        }

        public string StoreId
        {
            get { return _storeId; }
        }
    }

    [Assembler(typeof(MediaFileProviderAssembler))]
    public class MediaFileProviderData : DataProviderData
    {

        private const string _storeIdProperty = "storeId";
        [ConfigurationProperty(_storeIdProperty, IsRequired = true)]
        public string StoreId
        {
            get { return (string)base[_storeIdProperty]; }
            set { base[_storeIdProperty] = value; }
        }



        private const string _storeDescriptionProperty = "storeDescription";
        [ConfigurationProperty(_storeDescriptionProperty, IsRequired = true)]
        public string StoreDescription
        {
            get { return (string)base[_storeDescriptionProperty]; }
            set { base[_storeDescriptionProperty] = value; }
        }



        private const string _storeTitleProperty = "storeTitle";
        [ConfigurationProperty(_storeTitleProperty, IsRequired = true)]
        public string StoreTitle
        {
            get { return (string)base[_storeTitleProperty]; }
            set { base[_storeTitleProperty] = value; }
        }
    }


    internal class MediaFileProviderAssembler : IAssembler<IDataProvider, DataProviderData>
    {
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1062:ValidateArgumentsOfPublicMethods")]
        public IDataProvider Assemble(IBuilderContext context, DataProviderData objectConfiguration, IConfigurationSource configurationSource, ConfigurationReflectionCache reflectionCache)
        {
            var configuration = objectConfiguration as MediaFileProviderData;

            if (configuration == null) throw new ArgumentException("Expected configuration to be of type MediaFileDataProviderData", "objectConfiguration");

            

            return (IDataProvider)Activator.CreateInstance(objectConfiguration.Type, configuration.Name, configuration.StoreId, configuration.StoreTitle, configuration.StoreDescription);
        }

    }
}
